--[[ ArenaLive Spectator Functions: Spectated Unit Cache Handler
Created by: Vadrak
Creation Date: 14.02.2015
Last Update: "
Description: This is no actual ArenaLive handler, just a frame that is attached to the
			 ArenaLiveSpectator frame. It will track unit info for spectateda and
			 spectatedb unit types. Additionally, it will track how many players are
			 currently playing per team, because C_Commentator.GetNumPlayers() returns
			 values that do not concur with available players for the spectated unitIDs.
]]--

-- Addon Name and localisation table:
local addonName, L = ...;

--[[
**************************************************
******* GENERAL HANDLER SET UP STARTS HERE *******
**************************************************
]]--
ArenaLiveSpectator.UnitCache = CreateFrame("Frame", "ArenaLiveSpectatorUnitCache");
local UnitCache = ArenaLiveSpectator.UnitCache;
local unitCache = {};
local unitInfo = {};
local NUM_MAX_PLAYERS = 10;
local ELAPSED, ON_UPDATE_THROTTLE = 0, 0.5;
local numPlayers = {
	[1] = 0,
	[2] = 0,
};
UnitCache:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");



--[[
****************************************
****** HANDLER METHODS START HERE ******
****************************************
]]--
function UnitCache:Enable(statusID)
	self.statusID = statusID;
	UnitCache:PopulateUnitCacheTables();
	self:RegisterEvent("UNIT_PET");
	ELAPSED = ON_UPDATE_THROTTLE;
	self:Show();
	self.enabled = true;
	ArenaLive:Message("Enabling Spectated Unit Cache", "debug");
end

function UnitCache:Disable()
	self.statusID = nil;
	self:RegisterEvent("UNIT_PET");
	self:Hide();
	
	ELAPSED = 0;
	
	-- Reset unit info:
	table.wipe(unitCache);
	table.wipe(unitInfo);
	
	numPlayers[1] = 0;
	numPlayers[2] = 0;
	
	self.enabled = false;
	ArenaLive:Message("Disabling Spectated Unit Cache", "debug");
end

function UnitCache:IsEnabled()
	return self.enabled;
end

function UnitCache:GetNumPlayers(teamID)
	if ( type(teamID) ~= "number" ) then
		ArenaLive:Message(L["%s: Usage %s"], "error", "UnitCache:GetNumPlayers()", "UnitCache:GetNumPlayers(teamID)");
	elseif ( teamID < 1 or teamID > 2 ) then
		ArenaLive:Message(L["ArenaLiveSpectator.UnitCache:GetNumPlayers(): teamID too low or too high. Value must be 1 or 2."], "error");
	else
		return numPlayers[teamID]
	end
end

function UnitCache:GetUnitInfo(unit)
	if ( unit and unitInfo[unit] ) then
		return true, unit, unitInfo[unit].guid, unitInfo[unit].name, unitInfo[unit].team;
	else
		return nil;
	end
	
end

function UnitCache:GetUnitInfoByGUID(guid)
	local queryUnit;
	for unit, infoTable in pairs(unitInfo) do
		if ( guid == infoTable.guid ) then
			queryUnit = unit;
			break;
		end
	end
	
	return self:GetUnitInfo(queryUnit);
end

function UnitCache:GetUnitInfoByName(name)
	local queryUnit;
	for unit, infoTable in pairs(unitInfo) do
		if ( name == infoTable.name ) then
			queryUnit = unit;
			break;
		end
	end
	
	return self:GetUnitInfo(unit);
end

function UnitCache:CheckUnitState(unit)
	if ( not unit ) then
		return;
	end
	
	local unitName = UnitName(unit);
	local exists = UnitExists(unit);
	local guid = UnitGUID(unit);
	
	local state = ( exists and guid and unitName and unitName ~= UNKNOWN );
	if ( state ~= unitCache[unit] or ( unitInfo[unit] and guid ~= unitInfo[unit].guid ) ) then
		local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;
		
		local teamID;
		if ( unitType == "spectateda" ) then
			teamID = 1;
		elseif ( unitType == "spectatedb" ) then
			teamID = 2;
		end		
		
		local countMod;
		if ( state ) then
			countMod = 1;
		else
			countMod = -1;
		end	

		if ( state ~= unitCache[unit] and teamID ) then
			numPlayers[teamID] = numPlayers[teamID] + countMod;
			--ArenaLive:Message("%s: state = %s, countMod = %d, numPlayers[%d] = %d", "debug", unit, tostring(state), countMod, teamID, numPlayers[teamID]);
		end
		
		unitCache[unit] = state;
		--ArenaLive:Message([[UnitCache: Existence state or GUID changed for unit %s to %s. numPlayers[%d] = %d. Triggering "AL_SPEC_PLAYER_UPDATE" event...]], "debug", unit, tostring(state), teamID, numPlayers[teamID]);
		self:SetUnitInfo(unit);
		ArenaLive:TriggerEvent("AL_SPEC_PLAYER_UPDATE", unit, state);
	end
end

function UnitCache:PopulateUnitCacheTables()
	for i = 1, NUM_MAX_PLAYERS do
		local unit = "spectateda"..i;
		unitCache[unit] = false;
		unit = "spectatedb"..i;
		unitCache[unit] = false;
	end
end

function UnitCache:SetUnitInfo(unit)
	if ( unitCache[unit] ) then
		local name, realm = UnitName(unit);
		local guid = UnitGUID(unit);
		if ( not realm ) then
			realm = GetRealmName();
		end
		name = name.."-"..realm;
		local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;
		local teamID;
		local database = ArenaLive:GetDBComponent(addonName);
		if ( unitType == "spectateda" or unitType == "spectatedpeta" ) then
			teamID = 1; -- (Alliance or yellow team);
		elseif ( unitType == "spectatedb"  or unitType == "spectatedpetb" ) then
			teamID = 0; -- (Horde or green team);
		end
		
		unitInfo[unit] = {};
		unitInfo[unit].guid = guid;
		unitInfo[unit].name = name;
		unitInfo[unit].team = teamID;
	elseif ( unitInfo[unit] ) then
		table.wipe(unitInfo[unit]);
		unitInfo[unit] = nil;
	end
end

function UnitCache:OnEvent(event, ...)
	if ( event == "UNIT_PET" ) then
		local unit = ...;
		local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;
		if ( unitType == "spectateda" or unitType == "spectatedb" ) then
			local number = string.match(unit, "^[a-z]+([0-9]+)$");
			local unit;
			if ( unitType == "spectateda" ) then
				unit = "spectatedpeta"..number;
			else
				unit = "spectatedpetb"..number;
			end
			
			self:CheckUnitState(unit);
		end
	elseif ( event == "UPDATE_BATTLEFIELD_STATUS" ) then
		local statusID = ...;
		local status = GetBattlefieldStatus(statusID);
		local isSpectator = IsSpectator();
		if( status == "active" and isSpectator and ( not self.enabled or statusID ~= self.statusID ) ) then -- 
			self:Enable(statusID);
		elseif ( status == "none" and statusID == self.statusID ) then
			self:Disable();
		end
	end
end

function UnitCache:OnUpdate(elapsed)
	if ( self.enabled ) then
		ELAPSED = ELAPSED + elapsed;
		if ( ELAPSED >= ON_UPDATE_THROTTLE ) then
			ELAPSED = 0;
			for i = 1, NUM_MAX_PLAYERS do
				local unit = "spectateda"..i;
				UnitCache:CheckUnitState(unit);
				unit = "spectatedb"..i;
				UnitCache:CheckUnitState(unit);
			end
		end
	end
end

-- Disable on initial load:
UnitCache:Disable();
UnitCache:SetScript("OnEvent", UnitCache.OnEvent);
UnitCache:SetScript("OnUpdate", UnitCache.OnUpdate);